/* File: RoombaKarel.java
 * 
 * A Karel program in which Karel picks up all the beepers in a
 * square world.  Karel is assumed to start at (1, 1) and each
 * corner will have no more than one beeper on it.
 */

import stanford.karel.*;

public class RoombaKarel extends SuperKarel {
	public void run() {
		while (leftIsClear()) {
			sweepOneRow();
			moveToNextRow();
		}
		sweepOneRow();
	}
	
	/* Moves Karel up one row.
	 * 
	 * Precondition:  Karel is at the front of a row, facing East.
	 * Postcondition: Karel is at the front of the row above it, facing East.
	 */
	private void moveToNextRow() {
		turnLeft();
		move();
		turnRight();
	}
	
	/* Sweep up all of the beepers on the current row.
	 * 
	 * Precondition:  Karel is at the front of a row, facing east.
	 * Postcondition: Karel is at the front of that same row, still facing
	 *                east, and all the beepers in the row have been swept.
	 */
	private void sweepOneRow() {
		sweepToEnd();
		comeBackHome();
	}
	
	/* Returns Karel to the start of a row.
	 * 
	 * Precondition:  Karel is at the end of a row, facing east.
	 * Postcondition: Karel is at the start of the row, facing east.
	 */
	private void comeBackHome() {
		turnAround();
		while (frontIsClear()) {
			move();
		}
		turnAround();
	}
	
	/* Sweeps up the row, moving Karel to the end.
	 * 
	 * Precondition:  Karel is at the front of a row, facing east.
	 * Postcondition: Karel is at the end of that row, facing east,
	 *                -and- all the beepers in that row are swept.
	 * 
	 */
	private void sweepToEnd() {
		sweepCorner();
		while (frontIsClear()) {
			move();
			sweepCorner();
		}
	}
	
	/* Causes Karel to sweep up the corner that it's standing on.
	 * 
	 * Precondition:  None.
	 * Postcondition: Karel hasn't moved, but Karel has picked up a beeper
	 *                if there's one to pick up.
	 */
	private void sweepCorner() {
		if (beepersPresent()) {
			pickBeeper();
		}
	}
}